package com.mobvista.apptag.controller;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.JsonObject;
import com.mobvista.apptag.config.WebSecurityConfig;
import com.mobvista.apptag.entity.Feat;
import com.mobvista.apptag.entity.Storage;
import com.mobvista.apptag.entity.Tag;
import com.mobvista.apptag.entity.User;
import com.mobvista.apptag.mapper.FeatDao;
import com.mobvista.apptag.mapper.ResultDao;
import com.mobvista.apptag.mapper.TagDao;
import com.mobvista.apptag.mapper.UserDao;
import com.mobvista.apptag.service.FeatService;
import com.mobvista.apptag.service.StorageService;
import com.mobvista.apptag.utils.*;
import net.sf.json.JSONArray;
import net.sf.json.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author wangjf
 */
@Controller
@RequestMapping("/storage")
public class StorageController {

    @Autowired
    private ResultDao resultDao;
    @Autowired
    private UserDao userDao;
    @Autowired
    private TagDao tagDao;
    @Autowired
    private FeatDao featDao;
    @Autowired
    private FeatService featService;
    @Autowired
    private StorageService storageService;

    public static final Logger logger = LoggerFactory.getLogger(StorageController.class);

    @GetMapping("/edit")
    String edit(@RequestParam(name = "packageName") String packageName, Model model) {
        Tag tag = tagDao.find(packageName);
        if (StringUtils.isNotBlank(tag.getUrl()) && tag.getUrl().length() > 2 && tag.getUrl().startsWith("[") && tag.getUrl().endsWith("]")) {
            String url = tag.getUrl().substring(2, tag.getUrl().length() - 2).replace("\"", "").split(",")[0];
            tag.setUrl(url);
        } else {
            tag.setUrl("");
        }
        model.addAttribute("tag", tag);

        Storage result = storageService.find(packageName);
        model.addAttribute("resultCategory", JSONObject.fromObject(result.getTag()));
        List<Feat> categoryFeat = featDao.firstList("01");
        JSONArray jsonArray = JSONArray.fromObject(categoryFeat);
        model.addAttribute("firstCategory", jsonArray);
        List<Feat> secondCategoryFeat = featDao.list("01");
        JSONArray secondJsonArray = JSONArray.fromObject(secondCategoryFeat);
        model.addAttribute("secondCategory", secondJsonArray);
        List<Feat> ruleFeat = featDao.firstList("02");
        model.addAttribute("ruleCategory", JSONArray.fromObject(ruleFeat));
        List<Feat> styleFeat = featDao.firstList("03");
        model.addAttribute("styleCategory", JSONArray.fromObject(styleFeat));
        List<Feat> themeFeat = featDao.firstList("04");
        model.addAttribute("themeCategory", JSONArray.fromObject(themeFeat));
        return "storage/edit";
    }

    @PostMapping("/save")
    @ResponseBody
    R save(@SessionAttribute(WebSecurityConfig.SESSION_KEY) String userName,
           @RequestParam(name = "packageName") String packageName,
           @RequestParam(name = "categoryCode", required = false) String categoryCode,
           @RequestParam(name = "ruleCode", required = false) String ruleCode,
           @RequestParam(name = "styleCode", required = false) String styleCode,
           @RequestParam(name = "themeCode", required = false) String themeCode) {

        Map<String, String> codeMap = new HashMap<String, String>();
        Map<String, String> featMap = featService.getMap();

        // 类型标签
        if (StringUtils.isNotBlank(categoryCode)) {
            String[] codes = categoryCode.split(",");
            for (String code : codes) {
                codeMap.put(code, featMap.get(code));
            }
        }

        // 规则标签
        if (StringUtils.isNotBlank(ruleCode)) {
            String[] codes = ruleCode.split(",");
            for (String code : codes) {
                codeMap.put(code, featMap.get(code));
            }
        }

        // 风格标签
        if (StringUtils.isNotBlank(styleCode)) {
            String[] codes = styleCode.split(",");
            for (String code : codes) {
                codeMap.put(code, featMap.get(code));
            }
        }

        // 主题标签
        if (StringUtils.isNotBlank(themeCode)) {
            String[] codes = themeCode.split(",");
            for (String code : codes) {
                codeMap.put(code, featMap.get(code));
            }
        }
        ObjectMapper mapper = new ObjectMapper();
        String json = "";
        try {
            json = mapper.writeValueAsString(codeMap);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }

        if (storageService.update(packageName, userName, json)) {
            return R.ok();
        } else {
            return R.error(1, "保存失败！");
        }
    }

    @PostMapping("/list")
    @ResponseBody
    public PageUtil list(@RequestBody Query query, Model model) {
        //  PageHelper.startPage(query.getOffset() / query.getLimit() + 1, query.getLimit());
        List<Storage> results = storageService.list(query);
        String search = query.getSearch();
        int total = storageService.count(search);
        PageUtil pageUtil = new PageUtil(results, total);
        return pageUtil;
    }

    @GetMapping("")
    String storageTag(@SessionAttribute(WebSecurityConfig.SESSION_KEY) String username, Model model) {
        User user = userDao.find(username);
        model.addAttribute("userRole", user.getRole());
        return "storage/list";
    }

    @PostMapping("/upload")
    @ResponseBody
    public String upload(@RequestParam("file") MultipartFile multipartFile) {

        String rootPath = PropertyUtil.getProperty("app_tag.path");

        if (multipartFile.isEmpty()) {
            return "上传失败，请选择文件!";
        }
        String fileName = multipartFile.getOriginalFilename();
        File dest = new File(rootPath + "/" + fileName);
        if (dest.exists()) {
            dest.delete();
        }
        try {
            multipartFile.transferTo(dest);
            logger.info("上传成功");
            if (storageService.writeToMysql(dest)) {
                String execId = execAZ();
                if (StringUtils.isNotBlank(execId)) {
                    return "上传成功,execId == >> " + execId;
                } else {
                    return "上传失败！";
                }
            }
        } catch (IOException | URISyntaxException e) {
            logger.error(e.toString(), e);
        }
        return "上传失败！";
    }

    public static String execAZ() throws URISyntaxException {

        CloseableHttpClient client = HttpClients.createDefault();
        List<BasicNameValuePair> formparams = new ArrayList<>();

        String AZ_URL = PropertyUtil.getProperty("azkaban.server.http");
        final String serverUrl = AZ_URL + "azkaban/execute";
        URIBuilder uri = new URIBuilder();
        try {
            uri = new URIBuilder(serverUrl)
                    .addParameter("cluster", "offline")
                    .addParameter("projectName", "app_tag")
                    .addParameter("flowName", "mysql2hive");
        } catch (URISyntaxException e) {
            e.printStackTrace();
        }

        RequestConfig requestConfig = RequestConfig.custom()
                .setConnectTimeout(1000).setConnectionRequestTimeout(1000)
                .setSocketTimeout(1000).build();

        final HttpGet httpGet = new HttpGet(uri.build());

        String execId = "";
        CloseableHttpResponse response;
        try {
            response = client.execute(httpGet);
            BufferedReader rd = new BufferedReader(
                    new InputStreamReader(response.getEntity().getContent()));
            StringBuilder result = new StringBuilder();
            String line;
            while ((line = rd.readLine()) != null) {
                result.append(line);
            }
            JsonObject jsonObject = GsonUtil.String2JsonObject(result.toString());
            if (jsonObject.get("code").getAsInt() == 200) {
                execId = jsonObject.get("data").getAsJsonObject().get("execId").getAsString();
            }
        } catch (IOException e) {
            logger.info("Execute Failed!!!");
        } finally {
            httpGet.abort();
        }

        return execId;
    }
}