package com.mobvista.apptag.service.impl;

import com.github.pagehelper.PageHelper;
import com.mobvista.apptag.entity.TreeTag;
import com.mobvista.apptag.mapper.TreeTagDao;
import com.mobvista.apptag.service.TreeTagService;
import com.mobvista.apptag.utils.Query;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.*;
import java.util.*;
import java.util.regex.Pattern;

/**
 * @package: com.mobvista.apptag.service.impl
 * @author: wangjf
 * @date: 2020/12/3
 * @time: 2:15 下午
 * @email: jinfeng.wang@mobvista.com
 * @phone: 152-1062-7698
 */
@Service("treeTagService")
public class TreeTagServiceImpl implements TreeTagService {

    Pattern idIosPkgPtn = Pattern.compile("^id\\d+$");
    Pattern iosPkgPtn = Pattern.compile("^\\d+$");
    Pattern adrPkgPtn = Pattern.compile("^(?=^.{3,255}$)[-a-zA-Z0-9_][-a-zA-Z0-9_]{0,62}(\\.[a-zA-Z0-9_][-a-zA-Z0-9_]{0,62})+$");
    Pattern strPtn = Pattern.compile("^[-& a-zA-Z0-9]+$");

    @Autowired
    private TreeTagDao treeTagDao;

    @Override
    public List<TreeTag> list(Query query) {
        PageHelper.startPage(query.getOffset() / query.getLimit() + 1, query.getLimit());
        return treeTagDao.list(query.getSearch());
    }

    @Override
    public int count(Query query) {
        return treeTagDao.count(query.getSearch());
    }

    @Override
    public boolean writeToMysql(File file) throws IOException {
        String encoding = "UTF8";
        // 考虑到编码格式
        InputStreamReader read = new InputStreamReader(new FileInputStream(file), encoding);
        BufferedReader bufferedReader = new BufferedReader(read);
        String lineTxt;
        Set<TreeTag> set = new HashSet<>();
        while ((lineTxt = bufferedReader.readLine()) != null) {

            TreeTag treeTag = new TreeTag();
            String[] strings = lineTxt.replaceAll("\uFEFF", "").split(",", -1);
            if (strings.length > 1) {
                String packageName = strings[0].trim();
                String firstTag = "";
                if (StringUtils.isNotBlank(strings[1])) {
                    firstTag = strings[1];
                } else {
                    continue;
                }
                String secondTag = "";
                if (strings.length > 2 && StringUtils.isNotBlank(strings[2])) {
                    secondTag = strings[2];
                }
                String thirdTag = "";
                if (strings.length > 3 && StringUtils.isNotBlank(strings[3])) {
                    thirdTag = strings[3];
                }

                String platform = "";
                if (idIosPkgPtn.matcher(packageName).matches()) {
                    packageName = packageName.replace("id", "");
                }
                if (iosPkgPtn.matcher(packageName).matches()) {
                    platform = "ios";
                } else if (adrPkgPtn.matcher(packageName).matches()) {
                    platform = "android";
                }
                treeTag.setPackageName(packageName);
                treeTag.setPlatform(platform);
                if (strPtn.matcher(firstTag).matches()) {
                    treeTag.setFirstTag(firstTag);
                }
                if (strPtn.matcher(secondTag).matches()) {
                    treeTag.setSecondTag(secondTag);
                } else {
                    treeTag.setSecondTag("");
                }
                if (strPtn.matcher(thirdTag).matches()) {
                    treeTag.setThirdTag(thirdTag);
                } else {
                    treeTag.setThirdTag("");
                }
                treeTag.setUpdateTime(new Date());
                set.add(treeTag);
                if (idIosPkgPtn.matcher(packageName).matches()) {
                    TreeTag newTreeTag = update(treeTag, packageName.replace("id", ""));
                    set.add(newTreeTag);
                }
                if (iosPkgPtn.matcher(packageName).matches()) {
                    TreeTag newTreeTag = update(treeTag, "id" + packageName);
                    set.add(newTreeTag);
                }
            }
        }
        bufferedReader.close();
        read.close();
        if (batchInsert(new ArrayList<>(set))) {
            return true;
        }
        return false;
    }

    private TreeTag update(TreeTag oldTreeTag, String packageName) {
        TreeTag treeTag = new TreeTag();
        treeTag.setPackageName(packageName);
        treeTag.setFirstTag(oldTreeTag.getFirstTag());
        treeTag.setSecondTag(oldTreeTag.getSecondTag());
        treeTag.setThirdTag(oldTreeTag.getThirdTag());
        treeTag.setPlatform(oldTreeTag.getPlatform());
        treeTag.setUpdateTime(new Date());
        return treeTag;
    }

    public boolean batchInsert(List<TreeTag> list) {
        for (int i = 0; i <= list.size() / 1000; i++) {
            int startIndex = i * 1000;
            int endIndex;
            if ((i + 1) * 1000 > list.size()) {
                endIndex = list.size();
            } else {
                endIndex = (i + 1) * 1000;
            }
            if (!treeTagDao.batchInsert(list.subList(startIndex, endIndex))) {
                return false;
            }
        }
        return true;
    }
}
