package com.mobvista.apptag.utils;

import com.alibaba.fastjson.JSONObject;
import com.mobvista.apptag.main.AppTagClass;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.regex.Pattern;

/**
 * @package: com.mobvista.apptag.utils
 * @author: wangjf
 * @date: 2021/6/10
 * @time: 2:33 下午
 * @email: jinfeng.wang@mobvista.com
 */
public class UploadUtil {

    private static Logger logger = LoggerFactory.getLogger(UploadUtil.class);

    private static final String CONNECTION_URL = "jdbc:mysql://dataplatform-app-tag.c5yzcdreb1xr.us-east-1.rds.amazonaws.com:3306/app_tag?useUnicode=true&characterEncoding=utf8&useSSL=false";
    private static final String DRIVER = "com.mysql.jdbc.Driver";
    private static final String USER = "apptag_rw";
    private static final String PASSWORD = "7gyLEVtkER3u8c9";

    private static final SimpleDateFormat SDF = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    public static Map<String, AppTagClass> readTxtFileIntoStringSet(File file) {
        Map<String, String> tmap = getMap();
        Map<String, AppTagClass> map = new HashMap<>();
        try {
            if (file.isFile() && file.exists()) {
                InputStream read = new FileInputStream(file);
                String fileName = file.getName();
                Workbook workbook = null;
                if (fileName.toLowerCase().endsWith("xlsx")) {
                    workbook = new XSSFWorkbook(read);
                } else if (fileName.toLowerCase().endsWith("xls")) {
                    workbook = new HSSFWorkbook(read);
                }
                Sheet sheet = workbook.getSheetAt(0);
                List<List<String>> list = new ArrayList<>();
                Iterator<Row> rowIterator = sheet.iterator();
                while (rowIterator.hasNext()) {
                    AppTagClass appTagClass = new AppTagClass();
                    Row row = rowIterator.next();
                    List<String> rowList = new ArrayList<>();
                    Iterator<Cell> cellIterator = row.cellIterator();

                    String packageName = getValue(row.getCell(0));
                    String platform = getValue(row.getCell(1));
                    String tag = getValue(row.getCell(4));
                    if (StringUtils.isNotBlank(getValue(row.getCell(5)))) {
                        tag = getValue(row.getCell(5));
                    }
                    if (map.containsKey(packageName)) {
                        appTagClass = map.get(packageName);
                        Set<String> set = appTagClass.getTag_list();
                        set.add(tag);
                        appTagClass.setTag_list(set);
                    } else {
                        appTagClass.setPackage_name(packageName);
                        appTagClass.setApp_name("");
                        appTagClass.setPlatform(platform);
                        Set<String> set = new HashSet<>();
                        set.add(tag);
                        appTagClass.setTag_list(set);
                    }

                    String tagType = getValue(row.getCell(6));
                    String firstTag = getValue(row.getCell(7));
                    String secondTag = "";
                    if (StringUtils.isNotBlank(getValue(row.getCell(8)))) {
                        secondTag = getValue(row.getCell(8));
                    }
                    String tagId = "";
                    if (StringUtils.isNotBlank(secondTag)) {
                        if (tmap.containsKey((tagType + "-" + firstTag + "-" + secondTag).toUpperCase())) {
                            tagId = tmap.get((tagType + "-" + firstTag + "-" + secondTag).toUpperCase());
                        }
                    } else {
                        if (tmap.containsKey((tagType + "-" + firstTag).toUpperCase())) {
                            tagId = tmap.get((tagType + "-" + firstTag).toUpperCase());
                        }
                    }

                    if (StringUtils.isBlank(tagId)) {
                        continue;
                    }
                    String[] tags = tagId.split("#");
                    if (map.containsKey(packageName)) {
                        appTagClass = map.get(packageName);
                        JSONObject jsonObject = appTagClass.getTag_result();
                        jsonObject.put(tags[0], tags[1]);
                        appTagClass.setTag_result(jsonObject);
                    } else {
                        appTagClass.setPackage_name(packageName);
                        appTagClass.setApp_name("");
                        appTagClass.setPlatform(platform);
                        JSONObject jsonObject = new JSONObject();
                        jsonObject.put(tags[0], tags[1]);
                        appTagClass.setTag_result(jsonObject);
                    }
                    map.put(packageName, appTagClass);
                }
                read.close();
            } else {
                logger.info("找不到指定的文件");
            }
        } catch (Exception e) {
            logger.info("读取文件内容出错");
            e.printStackTrace();
        }
        return map;
    }

    public static String getValue(Cell oneCell) {
        String value = "";
        if (oneCell == null || oneCell.getCellTypeEnum() == CellType.BLANK
                || (oneCell.getCellTypeEnum() == CellType.STRING && oneCell.getStringCellValue().length() == 0)) {
            if (oneCell == null) {
            } else if (oneCell.getCellTypeEnum() == CellType.BLANK) {
            } else if (oneCell.getCellTypeEnum() == CellType.STRING && oneCell.getStringCellValue().length() == 0) {
            }
        } else {
            value = oneCell.getStringCellValue();
        }
        return value;
    }

    public static void insert(Set<AppTagClass> set) {
        try {
            Class.forName(DRIVER);
            Connection con = DriverManager.getConnection(CONNECTION_URL, USER, PASSWORD);
            // ON DUPLICATE KEY UPDATE tag = tag

            String insertSql = "REPLACE INTO tag_list(package_name, app_name, platform, category, tag, flag, install) VALUES @VALUES";
            StringBuilder stringBuilder = new StringBuilder();
            Iterator<AppTagClass> iset = set.iterator();
            while (iset.hasNext()) {
                AppTagClass appTagClass = iset.next();
                String platform;
                String packageName = appTagClass.package_name;
                String appName = appTagClass.app_name;
                String category = String.join(",", appTagClass.tag_list);
                Pattern iosPkgPtn = Pattern.compile("^id\\d+$");
                Pattern adrPkgPtn = Pattern.compile("^[0-9a-zA-Z\\.\\_\\-]+$");
                // Pattern otherAdrPkgPtn = Pattern.compile("^[\\_]+$");
                if (iosPkgPtn.matcher(packageName).matches() || adrPkgPtn.matcher(packageName).matches()) {
                    if (iosPkgPtn.matcher(packageName).matches()) {
                        platform = "IOS";
                        if (packageName != null && packageName.matches("^id\\d+$")) {
                            packageName = packageName.replace("id", "");
                        }
                    } else {
                        platform = "Android";
                    }
                    stringBuilder.append("('").append(packageName).append("','").append(appName).append("','")
                            .append(platform).append("','").append(category).append("','").append(1).append("','")
                            .append(1).append("',").append("30000000000),");
                }
            }

            String instring = insertSql.replace("@VALUES", stringBuilder.substring(0, stringBuilder.length() - 1));
            logger.info(instring);

            PreparedStatement stmt = con.prepareStatement(instring);
            stmt.execute();

            insertSql = "REPLACE INTO storage(package_name, app_name, platform, tag, update_time) VALUES @VALUES";
            stringBuilder = new StringBuilder();
            iset = set.iterator();
            logger.info("insert.size ==>> " + set.size());
            while (iset.hasNext()) {
                AppTagClass appTagClass = iset.next();
                String platform;
                String packageName = appTagClass.package_name;
                String appName = appTagClass.app_name;
                String tag = appTagClass.getTag_result().toJSONString();
                Pattern iosPkgPtn = Pattern.compile("^id\\d+$");
                Pattern adrPkgPtn = Pattern.compile("^[0-9a-zA-Z\\.\\_\\-]+$");
                // Pattern adrPkgPtn = Pattern.compile("^\\w+\\..*$");
                if (iosPkgPtn.matcher(packageName).matches() || adrPkgPtn.matcher(packageName).matches()) {
                    if (iosPkgPtn.matcher(packageName).matches()) {
                        platform = "IOS";
                        if (packageName != null && packageName.matches("^id\\d+$")) {
                            packageName = packageName.replace("id", "");
                        }
                    } else {
                        platform = "Android";
                    }
                    stringBuilder.append("('").append(packageName).append("','").append(appName).append("','")
                            .append(platform).append("','").append(tag).append("','").append(SDF.format(new Date())).append("'),");
                }
            }

            instring = insertSql.replace("@VALUES", stringBuilder.substring(0, stringBuilder.length() - 1));
            logger.info(instring);

            PreparedStatement stmt0 = con.prepareStatement(instring);
            stmt0.execute();
            con.close();
        } catch (Exception e) {
            logger.info(e.getMessage());
        }
    }

    public static Map<String, String> getMap() {
        String filePath = "/home/mobdev/workspace/AppTag/src/main/resources/datas/apptagId.txt";
        Map<String, String> map = new HashMap<>();
        try {
            String encoding = "UTF8";
            File file = new File(filePath);
            // 判断文件是否存在
            if (file.isFile() && file.exists()) {
                // 考虑到编码格式
                InputStreamReader read = new InputStreamReader(new FileInputStream(file), encoding);
                BufferedReader bufferedReader = new BufferedReader(read);
                String lineTxt;
                while ((lineTxt = bufferedReader.readLine()) != null) {
                    String[] appTags = lineTxt.split(";", -1);
                    String tagType = appTags[0];
                    String firstTag = appTags[1];
                    String secondTag = appTags[2];
                    String tagId;
                    String tagName = appTags[3];

                    if (StringUtils.isNotBlank(secondTag)) {
                        tagId = appTags[9];
                        map.put((tagType + "-" + firstTag + "-" + secondTag).toUpperCase(), tagId + "#" + tagName);
                    }
                    tagId = appTags[8];
                    map.put((tagType + "-" + firstTag).toUpperCase(), tagId + "#" + tagName);
                }
                bufferedReader.close();
                read.close();
            } else {
                logger.info("找不到指定的文件");
            }
        } catch (Exception e) {
            logger.info("读取文件内容出错");
            e.printStackTrace();
        }
        return map;
    }
}
